"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const extend = require("deep-extend");
const fs = require("fs");
const globby = require("globby");
const path = require("path");
const copy = require("recursive-copy");
const cache_1 = require("../services/cache");
const conf_1 = require("../services/conf");
const logger_1 = require("../services/logger");
const npm_1 = require("../services/npm");
const render_1 = require("../services/render");
const watcher_1 = require("../services/watcher");
const util_1 = require("../util");
const { resolve } = path;
const { RdTypes } = conf_1.default;
class DockerRun {
    constructor({ watch }) {
        this.watch = watch;
    }
    async start() {
        logger_1.spinner.start('Preparing Rde Runtime...');
        // generate rdt template from rdt chain
        const config = await this.composeRdcChain();
        // template render to .rde
        await this.renderDir(conf_1.default.tmpDir, config);
        // start watcher
        await this.createRuntime(config);
        logger_1.spinner.stop();
    }
    async composeRdcChain() {
        await util_1.default.asyncExec(`rm -rf ${conf_1.default.tmpDir} && mkdir ${conf_1.default.tmpDir}`);
        const { getRdcChain, getRdcConfFromChain, } = conf_1.default;
        const chain = getRdcChain('.');
        const rdcConf = getRdcConfFromChain(chain);
        let config = rdcConf;
        const rdaConfPath = resolve(conf_1.default.cwd, conf_1.default.appConfName);
        if (fs.existsSync(rdaConfPath)) {
            config = extend(rdcConf, require(rdaConfPath));
        }
        for (let node of chain) {
            let srcDir = resolve(node, 'template');
            await copy(srcDir, resolve(conf_1.default.tmpDir, 'template'), {
                overwrite: true,
                filter: ['**/node_modules'],
            });
        }
        await render_1.default.renderTo('module', {
            obj: config
        }, resolve(conf_1.default.tmpDir, conf_1.default.rdcConfName), {
            overwrite: true
        });
        return config;
    }
    async renderDir(dir, config) {
        const srcDir = resolve(dir, 'template');
        // @ts-ignore
        const { render: rdtRender, dev = { render: {} }, container } = config;
        const { includes } = rdtRender;
        const dataView = container ? container.render : dev ? dev.render : {};
        await render_1.default.renderDir(srcDir, Object.assign({}, dataView), includes, conf_1.default.runtimeDir, {
            overwrite: true,
            filter: ['**/node_modules'],
        });
        await util_1.default.asyncExec(`rm -rf ${dir}`);
    }
    async createRuntime(config) {
        const { cwd, runtimeDir } = conf_1.default;
        for (let { from, to, option } of config.mappings) {
            const appDir = resolve(cwd, from);
            const destDir = resolve(runtimeDir, to);
            await util_1.default.copy(appDir, destDir, { option });
        }
        await this.install();
        if (this.watch) {
            new watcher_1.default(config.mappings).start();
        }
    }
    async install() {
        /**
         * rde will trigger npm install under such condition:
         * 1. no cache info found
         * 2. no node_modules found
         * 3. rdc version changed if rdType is application
         * 4. any package.json under template dir has changed if rdType is container
         * 5. extends attr in rdc.config.js has changed if rdType is container
         */
        const { rdType, runtimeDir, cwd } = conf_1.default;
        const nodeModulesDir = resolve(runtimeDir, 'node_modules');
        const isApplication = rdType === RdTypes.Application;
        let rdcName;
        if (isApplication) {
            const rdaConf = conf_1.default.getAppConf();
            rdcName = rdaConf.container.name || '';
        }
        const isContainer = rdType === RdTypes.Container;
        let pkgJsonLMTs;
        let extendsName;
        if (isContainer) {
            const pkgJsonPaths = await globby(['**/package.json'], {
                cwd: resolve(cwd, 'template'),
                ignore: ['node_modules'],
            });
            // last-modified-time
            let lmt = '';
            pkgJsonPaths.forEach(pkg => {
                const pkgPath = resolve(cwd, 'template', pkg);
                lmt += fs.statSync(pkgPath).mtimeMs;
            });
            pkgJsonLMTs = lmt;
            const rdcConfPath = resolve(cwd, conf_1.default.rdcConfName);
            const rdcConf = require(rdcConfPath);
            extendsName = rdcConf.extends;
        }
        if (!cache_1.default.exist ||
            !fs.existsSync(nodeModulesDir) ||
            (isApplication && rdcName !== cache_1.default.get('rda.container')) ||
            (isContainer && pkgJsonLMTs !== cache_1.default.get('rdc.pkg.lmts')) ||
            (isContainer && extendsName && extendsName !== cache_1.default.get('rdc.extends'))) {
            await npm_1.default.install('', false, conf_1.default.runtimeDir);
            cache_1.default.set('rda.container', rdcName);
            cache_1.default.set('rdc.pkg.lmts', pkgJsonLMTs);
            cache_1.default.set('rdc.extends', extendsName);
        }
    }
}
exports.default = DockerRun;
